;**********************************************************************
;    Filename:	fm.asm 
;    Date:          	2004/11/23
;    File Version:
;**********************************************************************

	list      		p=16F88			 ; list directive to define processor
	#include		 <p16F88.inc>		 ; processor specific variable definitions

	__CONFIG   _CONFIG1,  _CP_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_IO & _LVP_OFF  & _MCLR_OFF 

;***** VARIABLE DEFINITIONS
w_temp			EQU     0x20			 ; variable used for context saving 
status_temp		EQU     0x21			 ; variable used for context saving

wait_data		EQU	0x30		;wait data ram
taihi			EQU	0x31		;taihi data
ch1_data		EQU	0x32		;CH1 Voltage  EEPROM 10H
ch2_data		EQU	0x33		;CH2 Voltage  EEPROM 11H
ch3_data		EQU	0x34		;CH3 Voltage  EEPROM 12H
ch4_data		EQU	0x35		;CH4 Voltage  EEPROM 13H
ch5_data		EQU	0x36		;CH5 Voltage  EEPROM 14H
ch6_data		EQU	0x37		;CH6 Voltage  EEPROM 15H

wait_data_2		EQU	0x39		;wait data 2
key_data			EQU	0x3a		;key data ram
									;key_data,0  CH1
									;key_data,1  CH2
									;key_data,2  CH3
key_input		EQU	0x3b		;key input ram
									;key_input,0  CH1
									;key_input,1  CH2
									;key_input,2  CH3
in_data			EQU	0x3c		;A/D in data
in_sub			EQU	0x3d		;A/D in sub data

ee_data			EQU	0x3f		;EEPROM data

ch1_sub			EQU	0x42		;CH1 Voltage sub EEPROM 18H
ch2_sub			EQU	0x43		;CH2 Voltage sub EEPROM 19H
ch3_sub			EQU	0x44		;CH3 Voltage sub EEPROM 1AH
ch4_sub			EQU	0x45		;CH4 Voltage sub EEPROM 1BH
ch5_sub			EQU	0x46		;CH5 Voltage sub EEPROM 1CH
ch6_sub			EQU	0x47		;CH6 Voltage sub EEPROM 1DH

tm0_setdata		EQU	0x63		;FFH-63H=156 156*128=19.968msec

;**********************************************************************
		ORG     		0x000             ; processor reset vector
  		goto    		main              ; go to beginning of program

;----------------------------------------------------------------------------------
		ORG     		0x004             ; interrupt vector location
		movwf   	w_temp            ; save off current W register contents
		movf		STATUS,w          ; move status register into W register
		movwf		status_temp       ; save off contents of STATUS register

		;---------------------------------	
		bcf			INTCON,TMR0IF
		movlw		tm0_setdata
		movwf		TMR0


next_int
		movf    		status_temp,w     ; retrieve copy of STATUS register
		movwf		STATUS            ; restore pre-isr STATUS register contents
		swapf   		w_temp,f
		swapf  		 w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;---------------------------------------------------------------------------------------
main
		call			osc_set
		call			port_set
		call			ad_set
		
		movlw		0ffh
		movwf		PORTA
		movlw		00h
		movwf		PORTB

		call			timer0_set

		call			shoki_set
		goto		ch1_read

;--------------------------------------
main_loop
		movlw		0ffh				;key input clear
		movwf		key_input
		btfss		PORTA,1		;CH1 sw check
		bcf			key_input,0		;CH1 sw input ari
		btfss		PORTA,2		;CH2 sw check
		bcf			key_input,1		;CH2 sw input ari		
		btfss		PORTA,3		;CH3 sw check
		bcf			key_input,2		;CH3 sw input ari		
		;--------------------------------- CH0 AD start
		movlw		41h
		movwf		ADCON0
		call			wait_20
		call			wait_20
		bsf			ADCON0,2
ch0_wait
		btfsc		ADCON0,2
		goto		ch0_wait
		movf		ADRESH,0
		movwf		in_data
		movf		ADRESL,0
		movwf		in_sub
		;----------------------------------CH1 sw check
		btfss		PORTA,1		;CH1 sw key check
		goto		ch1_sw_next
		bsf			key_data,0		;CH1 sw off
		goto		ch1_sw_end
ch1_sw_next
		btfsc		key_input,0		;CH1 sw input check
		goto		ch1_sw_end
		btfss		key_data,0		;CH1 sw on check
		goto		ch1_sw_end
		bcf			key_data,0		;CH1 sw on
		;------------------------
		btfss		PORTA,6
		goto		ch4_sw
		btfsc		PORTA,5
		goto		ch1_read
		movf		in_data,0
		movwf		ch1_data
		movf		in_sub,0
		movwf		ch1_sub
		call			eeprom_set
		goto		sw_end
ch1_read
		movf		ch1_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch1_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch1_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch4_sw
		btfsc		PORTA,5
		goto		ch4_read
		movf		in_data,0
		movwf		ch4_data
		movf		in_sub,0
		movwf		ch4_sub
		call			eeprom_set
		goto		sw_end
ch4_read
		movf		ch4_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch4_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch4_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch1_sw_end
		;----------------------------------CH2 sw check
		btfss		PORTA,2		;CH2 sw key check
		goto		ch2_sw_next
		bsf			key_data,1		;CH2 sw off
		goto		ch2_sw_end
ch2_sw_next
		btfsc		key_input,1		;CH2 sw input check
		goto		ch2_sw_end
		btfss		key_data,1		;CH2 sw on check
		goto		ch2_sw_end
		bcf			key_data,1		;CH2 sw on
		;------------------------
		btfss		PORTA,6
		goto		ch5_sw
		btfsc		PORTA,5
		goto		ch2_read
		movf		in_data,0
		movwf		ch2_data
		movf		in_sub,0
		movwf		ch2_sub
		call			eeprom_set
		goto		sw_end
ch2_read
		movf		ch2_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch2_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch2_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch5_sw
		btfsc		PORTA,5
		goto		ch5_read
		movf		in_data,0
		movwf		ch5_data
		movf		in_sub,0
		movwf		ch5_sub
		call			eeprom_set
		goto		sw_end
ch5_read
		movf		ch5_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch5_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch5_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch2_sw_end
		;----------------------------------CH3 sw check
		btfss		PORTA,3		;CH3 sw key check
		goto		ch3_sw_next
		bsf			key_data,2		;CH3 sw off
		goto		ch3_sw_end
ch3_sw_next
		btfsc		key_input,2		;CH3 sw input check
		goto		ch3_sw_end
		btfss		key_data,2		;CH3 sw on check
		goto		ch3_sw_end
		bcf			key_data,2		;CH3 sw on
		;------------------------
		btfss		PORTA,6
		goto		ch6_sw
		btfsc		PORTA,5
		goto		ch3_read
		movf		in_data,0
		movwf		ch3_data
		movf		in_sub,0
		movwf		ch3_sub
		call			eeprom_set
		goto		sw_end
ch3_read
		movf		ch3_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch3_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch3_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch6_sw
		btfsc		PORTA,5
		goto		ch6_read
		movf		in_data,0
		movwf		ch6_data
		movf		in_sub,0
		movwf		ch6_sub
		call			eeprom_set
		goto		sw_end
ch6_read
		movf		ch6_data,0
		movwf		PORTB
		bcf			PORTA,4
		nop
		bcf			PORTA,7
		nop
		btfsc		ch6_sub,6
		bsf			PORTA,7
		nop
		btfsc		ch6_sub,7
		bsf			PORTA,4
		nop
		goto		sw_end
ch3_sw_end

sw_end
	
;---------------------------------------
		goto		main_loop
;-------------------------------------------------------------------------------
wait_20
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		return
;-------------------------------------------------------------------------------
wait_10
		movlw		5h
		movwf		wait_data_2
wait_10_2
		call			wait_1
		decfsz		wait_data_2,1
		goto		wait_10_2
		return

wait_0
		movlw		0ffh
		movwf		wait_data
		goto		wait_2
wait_1		
		movlw		0ffh
		movwf		wait_data
wait_2
		nop
		decfsz		wait_data,1
		goto		wait_2
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
shoki_set
		movlw		00h
		call			eeprom_read
		sublw		96h				;HANTEI DATA
		btfss		STATUS,2
		goto		shoki_set2
		movlw		10h
		call			eeprom_read
		movwf		ch1_data
		movlw		11h
		call			eeprom_read
		movwf		ch2_data
		movlw		12h
		call			eeprom_read
		movwf		ch3_data
		movlw		13h
		call			eeprom_read
		movwf		ch4_data
		movlw		14h
		call			eeprom_read
		movwf		ch5_data
		movlw		15h
		call			eeprom_read
		movwf		ch6_data
		
		movlw		18h
		call			eeprom_read
		movwf		ch1_sub
		movlw		19h
		call			eeprom_read
		movwf		ch2_sub
		movlw		1Ah
		call			eeprom_read
		movwf		ch3_sub
		movlw		1Bh
		call			eeprom_read
		movwf		ch4_sub
		movlw		1Ch
		call			eeprom_read
		movwf		ch5_sub
		movlw		1Dh
		call			eeprom_read
		movwf		ch6_sub
		return

shoki_set2
		movlw		00h
		movwf		ch1_data
		movwf		ch2_data
		movwf		ch3_data
		movwf		ch4_data
		movwf		ch5_data
		movwf		ch6_data
		
		movwf		ch1_sub
		movwf		ch2_sub
		movwf		ch3_sub
		movwf		ch4_sub
		movwf		ch5_sub
		movwf		ch6_sub
		return
;-------------------------------------------------------------------------------
eeprom_set
		movf		ch1_data,0
		movwf		ee_data
		movlw		10h
		call			eeprom_write
		movf		ch2_data,0
		movwf		ee_data
		movlw		11h
		call			eeprom_write
		movf		ch3_data,0
		movwf		ee_data
		movlw		12h
		call			eeprom_write
		movf		ch4_data,0
		movwf		ee_data
		movlw		13h
		call			eeprom_write
		movf		ch5_data,0
		movwf		ee_data
		movlw		14h
		call			eeprom_write
		movf		ch6_data,0
		movwf		ee_data
		movlw		15h
		call			eeprom_write
		
		movf		ch1_sub,0
		movwf		ee_data
		movlw		18h
		call			eeprom_write
		movf		ch2_sub,0
		movwf		ee_data
		movlw		19h
		call			eeprom_write
		movf		ch3_sub,0
		movwf		ee_data
		movlw		1Ah
		call			eeprom_write
		movf		ch4_sub,0
		movwf		ee_data
		movlw		1Bh
		call			eeprom_write
		movf		ch5_sub,0
		movwf		ee_data
		movlw		1Ch
		call			eeprom_write
		movf		ch6_sub,0
		movwf		ee_data
		movlw		1Dh
		call			eeprom_write		
		
eeprom_id
		movlw		96h
		movwf		ee_data
		movlw		00h
		call			eeprom_write
		return

eeprom_read
		call			banksel_2
		movwf		EEADR
		call			banksel_3
		bsf			EECON1,RD
		call			banksel_2
		movf		EEDATA,0
		call			banksel_0		
		return

eeprom_write
		call			banksel_2
		movwf		EEADR
		call			banksel_0
		movf		ee_data,0
		call			banksel_2
		movwf		EEDATA
		call			banksel_3
		bsf			EECON1,WREN
		movlw		55h
		movwf		EECON2
		movlw		0aah
		movwf		EECON2
		bsf			EECON1,WR
eeprom_write2
		btfsc		EECON1,WR
		goto		eeprom_write2
		call			banksel_0
		return
;------------------------------------------------------------------------------
banksel_0
		bcf			STATUS,RP0
		nop
		bcf			STATUS,RP1
		nop
		return
banksel_1
		bsf			STATUS,RP0
		nop
		bcf			STATUS,RP1
		nop
		return
banksel_2
		bcf			STATUS,RP0
		nop
		bsf			STATUS,RP1
		nop
		return
banksel_3
		bsf			STATUS,RP0
		nop
		bsf			STATUS,RP1
		nop
		return
;------------------------------------------------------------------------------
osc_set
		call			banksel_1
								;(0)NC
								;(110)IRCF2-IRCF0
								;(0)OSTS
								;(1)IOFS
								;(00)SCS
		movlw		64h
		movwf		OSCCON
		call			banksel_0
		return
;-------------------------------------------------------------------------------
port_set							;RA0,RA1,RA2,RA3,RA5,RA6(IN)
								;RA4,RA7(OUT)
								;RB0,RB1,RB2,RB3,RB4,RB5,RB6,RB7(OUT)
		call			banksel_1
		clrf			TRISA
		bsf			TRISA,0
		nop
		bsf			TRISA,1
		nop
		bsf			TRISA,2
		nop
		bsf			TRISA,3
		nop
		bsf			TRISA,5
		nop
		bsf			TRISA,6
		nop
		clrf			TRISB
		call			banksel_0
		return
;------------------------------------------------------------------------------
ad_set
								;(01)ADCS1,ADCS0
								;(000)CHS2,CHS1,CHS0
								;(0)GO/DONE
								;(0)NC
								;(1)ADON
		movlw		41h
		movwf		ADCON0
		call			banksel_1
								;(0)ADFM
								;(1)ADCS2
								;(00)VCFG1,VCFG0
								;(0000)
		movlw		040h
		movwf		ADCON1
								;(0)NC
								;(0000001)ANS6-ANS0
		movlw		01h
		movwf		ANSEL
		call			banksel_0
		return
;------------------------------------------------------------------------------
timer0_set

		call			banksel_1
		movlw		086h		;(1)PORTB NOT PULLUP
								;(0)INTEDG
								;(0)TOCS
								;(0)TOSE
								;(0)PSA
								;(110)1/128
		movwf		OPTION_REG
		call			banksel_0

		movlw		tm0_setdata
		movwf		TMR0
		bcf			INTCON,TMR0IE
		bcf			INTCON,GIE
		return
;-------------------------------------------------------------------------------

		end                     ; directive 'end of program'
